#ifndef INC_MYTESTSERVER_H
#define INC_MYTESTSERVER_H
//-----------------------------------------------------------------------------
/*!
\file MyTestServer.h
\brief
\if english
  Header for MyTestServer.
\endif

\if german
  Header für MyTestServer.
\endif
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTcpLib module.

EuTcpLib is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "TcpTypes.h"
#include "TcpServer.h"
//-----------------------------------------------------------------------------
/*!
\brief
\if english
  An example class for a simple server derived from Server.

  Messages from connected clients are output. Additionally, MyTestServer can
  send messages to all clients.

  Implementation of MyTestServer:
  
  <B>MyTestServer.cpp</B>
  \include MyTestServer.cpp

  \anchor TCP_MYTESTSTSERVER_USING
  The MyTestServer class can be tried out as follows:
\endif

\if german
  Eine Beispielklasse für einen einfachen Server abgeleitet von Server.

  Es werden Nachrichten von angeschlossenen Clients ausgegeben. Zusätzlich
  kann MyTestServer Nachrichten an alle Clients versenden.

  Implementierung von MyTestServer:
  <B>MyTestServer.cpp</B>
  \include MyTestServer.cpp

  \anchor TCP_MYTESTSERVER_USING
  Die Klasse MyTestServer kann folgendermassen ausprobiert werden:
\endif

<B>ServerMain.cpp</B>
\include ServerMain.cpp

\copyright © 2010 Helmut Jakoby
\author Helmut Jakoby
*/
class MyTestServer : public tcp::Server
{
  protected:
    //============== Konstruktor
    //-------------------------------------------------------------------------
    /*!
    \if english
      This constructor is not available.
    \endif

    \if german
      Dieser Konstruktor steht niccht zur Verfügung.
    \endif
    */
    MyTestServer();
    //-------------------------------------------------------------------------
    /*!
    \if english
      This constructor is not available.
    \endif

    \if german
      Dieser Konstruktor steht niccht zur Verfügung.
    \endif
    */
    MyTestServer( unsigned int,
                  int,
                  int,
                  int,
                  int );
    //-------------------------------------------------------------------------
    /*!
    \if english
      This constructor is not available.
    \endif

    \if german
      Dieser Konstruktor steht niccht zur Verfügung.
    \endif
    */
    MyTestServer( unsigned int ,
                  int,
                  int,
                  int,
                  unsigned int,
                  int );
    //-------------------------------------------------------------------------

  public:
    //============== Konstruktoren
    //-------------------------------------------------------------------------
    /*!
    \if english
      The constructor starts the server.
      \param[in] uiPort The port of the server.
      \param[in] iAdressFamily The socket address family to instantiate a socket
        (usually AF_INET ).
      \param[in] iSocketType The socket type (usually SOCK_STREAM ).
      \param[in] iProtocol The socket protocol (usually 0).
      \param[in] uiDataSize The packet size.
      \param[in] ulMaxClients The maximum number of clients allowed to connect.
      \param[in] iSecTimeOut The time until waiting for a connection or waiting
        for data is aborted.
      \exception eut::ErrorNException An exception is thrown if an error occurs.
    \endif

    \if german
      Der Konstruktor startet den Server.
      \param[in] uiPort Der Port des Servers.
      \param[in] iAdressFamily Die Socket-AdressFamily um einen Socket zu
        instanziieren (i.d.R. AF_INET ).
      \param[in] iSocketType Der Socket-Typ (i.d.R. SOCK_STREAM ).
      \param[in] iProtocol Das Socket-Protokol (i.d.R. 0).
      \param[in] uiDataSize Die Paketgröße.
      \param[in] ulMaxClients Die maximale Anzahl von Clients, die sich verbinden
        dürfen.
      \param[in] iSecTimeOut Die Zeit bis ein Warten auf eine Verbindung bzw.
        warten auf Daten abgebrochen wird.
      \exception eut::ErrorNException Bei einem Fehler wird eine Exception geworfen.
    \endif
    */
    MyTestServer( unsigned int uiPort,
                  int iAdressFamily,
                  int iSocketType,
                  int iProtocol,
                  unsigned int uiDataSize,
                  unsigned long ulMaxClients,
                  int iSecTimeOut );
    //-------------------------------------------------------------------------

     //============== Destruktore
    //-------------------------------------------------------------------------
    /*!
    \if english
      Destructor.
    \endif

    \if german
      Destruktor.
    \endif
    */
    virtual ~MyTestServer();
    //-------------------------------------------------------------------------

  private:
    //============== Copy-Konstruktor
    //-------------------------------------------------------------------------
    /*!
    \if english
      The copy constructor is not available.
    \endif

    \if german
      Der Copy-Konstruktor steht nicht zur Verfügung.
    \endif
    */
    MyTestServer( const MyTestServer & );
    //-------------------------------------------------------------------------

  public:
    //============== Methoden
    //-------------------------------------------------------------------------
    /*!
    \if english
      Outputs a message from a connected client via std::cout.
      \param[in] pTcpMessage The message to be processed from the server.
      \return A return < 0 indicates an error.
    \endif

    \if german
      Gibt eine Nachricht von einem verbundenen Client über std::cout aus.
      \param[in] pTcpMessage Die zu verarbeitende Nachricht vom Server.
      \return Eine Rückgabe < 0 zeigt einen Fehler an.
    \endif
    */
    virtual void processClientMessage( tcp::AbstractMessage * pTcpMessage );
    //-------------------------------------------------------------------------
    /*!
    \if english
      Sends a text message to all connected clients.
      \param[in] strMsg The message to be sent.
      \return A return < 0 indicates an error.
    \endif

    \if german
      Sendet an alle verbundenen Clients eine Text-Nchricht.
      \param[in] strMsg Die zu versendende Nachricht.
      \return Eine Rückgabe < 0 zeigt einen Fehler an.
    \endif
    */
    int sendToAllClients( std::string strMsg );
    //-------------------------------------------------------------------------

  protected:
    //============== Methoden
    //-------------------------------------------------------------------------
    /*!
    \if english
      Inserts a new client as 'ServerReceiverTransmitter' in its m_ClientMap.
      After the same call in the superclass, displays the client here via
      std::cout.
      \param[in] pNewServerReceiverTransmitter The client is in this object.
    \endif

    \if german
      Fügt einen neuen Client als 'ServerReceiverTransmitter' in seine m_ClientMap ein.
      Zeigt nach selbigen Aufruf in Oberklasse den Client hier über std::cout an.
      \param[in] pNewServerReceiverTransmitter Der Client ist in diesem Objekt.
    \endif
    */
    virtual void insertNewClient( Server::ServerReceiverTransmitter * pNewServerReceiverTransmitter );
    //-------------------------------------------------------------------------

  private:
    //============== Operatoren
    //-------------------------------------------------------------------------
    /*!
    \if english
      The assignment operator is not available.
    \endif

    \if german
      Der Zuweisungsoperator steht nicht zur Verfügung.
    \endif
    */
    MyTestServer & operator= ( const MyTestServer & );
    //-------------------------------------------------------------------------
};
//-----------------------------------------------------------------------------
#endif
