#ifndef INC_TCPERR_H
#define INC_TCPERR_H
/*!
\file TcpErr.h
\brief
\if english
  For each library, here 'TcpLib' there is an error file.
\endif

\if german
  Für jede Bibliothek, hier 'TcpLib' gibt es eine Fehler-Datei.
\endif
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTcpLib module.

EuTcpLib is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
namespace tcp
{
  //-----------------------------------------------------------------------------
  /*!
  \enum ERRORS
  \brief
  \if english
    Error range = -10001 to -10200

    For the socket errors (all from -10004 to -11004) descriptions can be found on
    the Internet.

    For Windows Sockets e.g. under:
    https://msdn.microsoft.com/de-de/library/windows/desktop/ms740668%28v=vs.85%29.aspx
    or generally also under: https://gist.github.com/gabrielfalcao/4216897

    This list is not complete!
  \endif

  \if german
    Fehlerbereich = -10001 bis -10200

    Für die Socket-Fehler (alle von -10004 bis -11004) sind Beschreibungen im
    Internet zu finden.

    Für Windows Sockets z.B. unter:
    https://msdn.microsoft.com/de-de/library/windows/desktop/ms740668%28v=vs.85%29.aspx
    oder allgemein auch unter: https://gist.github.com/gabrielfalcao/4216897

    Aufzählung nicht vollständig!
  \endif
  */
  enum ERRORS
  {
    //-------------------------------------------------------------------------
    /*! -10004: */
    ERR_WSAEINTR = -10004,
    //-------------------------------------------------------------------------
    /*! -10009: */
    ERR_WSAEBADF = -10009,
    //-------------------------------------------------------------------------
    /*! -10013: */
    ERR_WSAEACCES = -10013,
    //-------------------------------------------------------------------------
    /*! -10014: */
    ERR_WSAEFAULT = -10014,
    //-------------------------------------------------------------------------
    /*! -10022: */
    ERR_WSAEINVAL = -10022,
    //-------------------------------------------------------------------------
    /*! -10024: */
    ERR_WSAEMFILE = -10024,
    //-------------------------------------------------------------------------
    /*! -10035: */
    ERR_WSAEWOULDBLOCK = -10035,
    //-------------------------------------------------------------------------
    /*! -10036: */
    ERR_WSAEINPROGRESS = -10036,
    //-------------------------------------------------------------------------
    /*! -10037: */
    ERR_WSAEALREADY = -10037,
    //-------------------------------------------------------------------------
    /*! -10038: */
    ERR_WSAENOTSOCK = -10038,
    //-------------------------------------------------------------------------
    /*! -10039: */
    ERR_WSAEDESTADDRREQ = -10039,
    //-------------------------------------------------------------------------
    /*! -10040: */
    ERR_WSAEMSGSIZE = -10040,
    //-------------------------------------------------------------------------
    /*! -10041: */
    ERR_WSAEPROTOTYPE = -10041,
    //-------------------------------------------------------------------------
    /*! -10042: */
    ERR_WSAENOPROTOOPT = -10042,
    //-------------------------------------------------------------------------
    /*! -10043: */
    ERR_WSAEPROTONOSUPPORT = -10043,
    //-------------------------------------------------------------------------
    /*! -10044: */
    ERR_WSAESOCKTNOSUPPORT = -10044,
    //-------------------------------------------------------------------------
    /*! -10045: */
    ERR_WSAEOPNOTSUPP = -10045,
    //-------------------------------------------------------------------------
    /*! -10046: */
    ERR_WSAEPFNOSUPPORT = -10046,
    //-------------------------------------------------------------------------
    /*! -10047: */
    ERR_WSAEAFNOSUPPORT = -10047,
    //-------------------------------------------------------------------------
    /*! -10048: */
    ERR_WSAEADDRINUSE = -10048,
    //-------------------------------------------------------------------------
    /*! -10049: */
    ERR_WSAEADDRNOTAVAIL = -10049,
    //-------------------------------------------------------------------------
    /*! -10050: */
    ERR_WSAENETDOWN = -10050,
    //-------------------------------------------------------------------------
    /*! -10051: */
    ERR_WSAENETUNREACH = -10051,
    //-------------------------------------------------------------------------
    /*! -10052: */
    ERR_WSAENETRESET = -10052,
    //-------------------------------------------------------------------------
    /*! -10053: */
    ERR_WSAECONNABORTED = -10053,
    //-------------------------------------------------------------------------
    /*! -10054: */
    ERR_WSAECONNRESET = -10054,
    //-------------------------------------------------------------------------
    /*! -10055: */
    ERR_WSAENOBUFS = -10055,
    //-------------------------------------------------------------------------
    /*! -10056: */
    ERR_WSAEISCONN = -10056,
    //-------------------------------------------------------------------------
    /*! -10057: */
    ERR_WSAENOTCONN = -10057,
    //-------------------------------------------------------------------------
    /*! -10058: */
    ERR_WSAESHUTDOWN = -10058,
    //-------------------------------------------------------------------------
    /*! -10059: */
    ERR_WSAETOOMANYREFS = -10059,
    //-------------------------------------------------------------------------
    /*! -10060: */
    ERR_WSAETIMEDOUT = -10060,
    //-------------------------------------------------------------------------
    /*! -10061: */
    ERR_WSAECONNREFUSED = -10061,
    //-------------------------------------------------------------------------
    /*! -10062: */
    ERR_WSAELOOP = -10062,
    //-------------------------------------------------------------------------
    /*! -10063: */
    ERR_WSAENAMETOOLONG = -10063,
    //-------------------------------------------------------------------------
    /*! -10064: */
    ERR_WSAEHOSTDOWN = -10064,
    //-------------------------------------------------------------------------
    /*! -10065: */
    ERR_WSAEHOSTUNREACH = -10065,
    //-------------------------------------------------------------------------
    /*! -10066: */
    ERR_WSAENOTEMPTY = -10066,
    //-------------------------------------------------------------------------
    /*! -10067: */
    ERR_WSAEPROCLIM = -10067,
    //-------------------------------------------------------------------------
    /*! -10068: */
    ERR_WSAEUSERS = -10068,
    //-------------------------------------------------------------------------
    /*! -10069: */
    ERR_WSAEDQUOT = -10069,
    //-------------------------------------------------------------------------
    /*! -10070: */
    ERR_WSAESTALE = -10070,
    //-------------------------------------------------------------------------
    /*! -10071: */
    ERR_WSAEREMOTE = -10071,
    //-------------------------------------------------------------------------
    /*! -10091: */
    ERR_WSASYSNOTREADY = -10091,
    //-------------------------------------------------------------------------
    /*! -10092: */
    ERR_WSAVERNOTSUPPORTED = -10092,
    //-------------------------------------------------------------------------
    /*! -10093: */
    ERR_WSANOTINITIALISED = -10093,
    /*! -11001: */
    ERR_WSAHOST_NOT_FOUND = -11001,
    //-------------------------------------------------------------------------
    /*! -11002: */
    ERR_WSATRY_AGAIN = -11002,
    //-------------------------------------------------------------------------
    /*! -11003: */
    ERR_WSANO_RECOVERY = -11003,
    //-------------------------------------------------------------------------
    /*! -11004: */
    ERR_WSANO_DATA = -11004,
    //-------------------------------------------------------------------------
    /*! -10101:
    \if english
      A, not allowed, NULL parameter was passed.
    \endif

    \if german
      Es wurde ein, nicht erlaubter, NULL-Parameter übergeben.
    \endif
    */
    ERR_PARAMETER_NULL_FORBIDDEN = -10101,
    //-------------------------------------------------------------------------
    /*! -10102:
    \if english
      A socket to be initialized already exists or is already connected.
    \endif

    \if german
      Ein zu initialisierender Socket existiert bereits bzw. ist schon verbunden.
    \endif
    */
    ERR_SOCKET_ALREADY_EXISTING = -10102,
    //-------------------------------------------------------------------------
    /*! -10103:
    \if english
      The expected socket does not exist.
    \endif

    \if german
      Der erwarteter Socket existiert nicht.
    \endif
    */
    ERR_SOCKET_NOT_EXISTING = -10103,
    //-------------------------------------------------------------------------
    /*! -10104:
    \if english
      No host name was passed as parameter.
    \endif

    \if german
      Es wurde keinen Hostname als Parameter übergeben.
    \endif
    */
    ERR_NO_HOSTNAME = -10104,
    //-------------------------------------------------------------------------
    /*! -10105:
    \if english
      The host name is unknown.
    \endif

    \if german
      Der Hostname ist unbekannt.
    \endif
    */
    ERR_UNKNOWN_HOSTNAME = -10105,
    //-------------------------------------------------------------------------
    /*! -10106:
    \if english
      No or no valid port was passed as parameter.
    \endif

    \if german
      Es wurde kein bzw. kein gültiger Port als Parameter übergeben.
    \endif
    */
    ERR_NO_PORT = -10106,
    //-------------------------------------------------------------------------
    /*! -10107:
    \if english
      The socket connection already exists.
    \endif

    \if german
      Die Socketverbindung besteht schon.
    \endif
    */
    ERR_ALLREADY_CONNECTED = -10107,
    //-------------------------------------------------------------------------
    /*! -10108:
    \if english
      The socket connection was interrupted.
    \endif

    \if german
      Die Socketverbindung wurde unterbrochen.
    \endif
    */
    ERR_CONNECTION_BROKEN = -10108,
    //-------------------------------------------------------------------------
    /*! -10109:
    \if english
      An expected data size is NULL.
    \endif

    \if german
      Eine erwartete Datengröße ist NULL.
    \endif
    */
    ERR_NO_DATASIZE = -10109,
    //-------------------------------------------------------------------------
    /*! -10110:
    \if english
      The maximum number of connections was not set or is 0.
    \endif

    \if german
      Die maximale Anzahl von Verbindungen wurde nicht gesetzt bzw. ist 0.
    \endif
    */
    ERR_MAX_CLIENTS_TO_LOW = -10110,
    //-------------------------------------------------------------------------
    /*! -10111:
    \if english
      The Server has not found a client in its client list.
    \endif

    \if german
      Der Server hat einen Client in seiner Clientliste nicht gefunden.
    \endif
    */
    ERR_NO_SOCKETINDEX = -10111,
    //-------------------------------------------------------------------------
    /*! -10112:
    \if english
      A received stream does not have the expected format.
    \endif

    \if german
      Ein empfangener Stream hat nicht das erwartete Format.
    \endif
    */
    ERR_WRONG_STREAMTYPE = -10112,
    //-------------------------------------------------------------------------
    /*! -10113: */
    ERR_FREE_1 = -10113,
    //-------------------------------------------------------------------------
    /*! -10114: */
    ERR_FREE_2 = -10114,
    //-------------------------------------------------------------------------
    /*! -10115:
    \if english
      The Client has no Transmitter.
    \endif

    \if german
      Der Client hat keinen Transmitter.
    \endif
    */
    ERR_NO_TRANSMITTER = -10115,
    //-------------------------------------------------------------------------
    /*! -10116:
    \if english
      The Server has no ServerReceiver.
    \endif

    \if german
      Der Server hat keinen ServerReceiver.
    \endif
    */
    ERR_NO_RECEIVER = -10116,
    //-------------------------------------------------------------------------
    /*! -10117: */
    ERR_FREE_3 = -10117,
    //-------------------------------------------------------------------------
    /*! -10118:
    \if english
      The maximum number of connections was exceeded.
    \endif

    \if german
      Die maximale Anzahl von Verbindungen wurde überschritten.
    \endif
    */
    ERR_TO_MUCH_CLIENTS = -10118,
    //-------------------------------------------------------------------------
    /*! -10119:
    \if english
      A received stream has incorrect data.
    \endif

    \if german
      Ein empfangener Stream hat unkorrekte Daten.
    \endif
    */
    ERR_NO_CORRECT_STREAM_DATA = -10119,
    //-------------------------------------------------------------------------
    /*! -10120:
    \if english
      Message passed as parameter is not a server message.
    \endif

    \if german
      Als Parameter übergebene Nachricht ist keine Server-Nachricht.
    \endif
    */
    ERR_NO_SERVER_MSG = -10120,
    //-------------------------------------------------------------------------
    /*! -10121:
    \if english
      The waiting time for an answer has expired (timeout).
    \endif

    \if german
      Die Wartezeit auf eine Antwort ist abgelaufen (Zeitüberschreitung).
    \endif
    */
    ERR_TIME_OUT = -10121,
    //-------------------------------------------------------------------------
    /*! -10122:
    \if english
      The method call is prohibited.
    \endif

    \if german
      Der Methodenaufruf ist verboten.
    \endif
    */
    ERR_METHOD_FORBIDDEN = -10122,
    //-------------------------------------------------------------------------
    /*! -10123:
    \if english
      The peername request failed.
    \endif

    \if german
      Die Abfrage nach dem Peername schlug fehl.
    \endif
    */
    ERR_GETPEERNAME_FAILED = -10123,
    //-------------------------------------------------------------------------
    /*! -10200:
    \if english
      Unknown error.
    \endif

    \if german
      Unbekannter Fehler.
    \endif
    */
    ERR_UNKNOWN = -10200
  };
  //---------------------------------------------------------------------------
} // namespace tcp
//-----------------------------------------------------------------------------
#endif
