#ifndef INC_TCPRECEIVER_H
#define INC_TCPRECEIVER_H
//-----------------------------------------------------------------------------
/*!
\file TcpReceiver.h
\brief
\if english
  Header for Receiver.
\endif

\if german
  Header für TcpReceiver.
\endif
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTcpLib module.

EuTcpLib is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "TcpTypes.h"
#include "TcpReceiverTransmitter.h"

#if defined (__PTHREADS)
  #include "EuEndlessPThread.h"
#else
  #include "EuRepeatingThread.h"
#endif

#include "TcpMain.h"
#include <queue>

#if defined(_MSC_VER) || defined(__MINGW32__) || defined(__MINGW64__)
  #include <winsock.h>
#endif
//-----------------------------------------------------------------------------
namespace tcp
{
  //---------------------------------------------------------------------------
  //Forwards
  class Server;
  class AbstractMessage;
  class Client;
  //---------------------------------------------------------------------------
  /*!
  \brief
  \if english
    Superclass for all TcpReceiver that communicate via a SOCKET and deliver their
    receive result to their parent object.
  \endif

  \if german
    Oberklasse für alle TcpReceiver, welche über ein SOCKET kommunizieren und
    ihr Empfangsergebnis an ihr Elternobjekt liefert.
  \endif
  \sa #m_pParentTcpMain

  \copyright © 2010 Helmut Jakoby
  \author Helmut Jakoby
  */
  class __tcp_export_dll Receiver : public ReceiverTransmitter
  {
    protected:
      //============== Attribute
      //-----------------------------------------------------------------------
      /*!
      \if english
        The parent object (Client or Server)
      \endif

      \if german
        Das Elternobjekt (Client oder Server).
      \endif
      */
      TcpMain * m_pParentTcpMain;
      //-----------------------------------------------------------------------

      //============== Konstruktoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor is not available.
      \endif

      \if german
        Dieser Konstruktor steht niccht zur Verfügung.
      \endif
      */
      Receiver();
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor is not available.
      \endif

      \if german
        Dieser Konstruktor steht niccht zur Verfügung.
      \endif
      */
      Receiver( SOCKET Socket );
      //-----------------------------------------------------------------------


    public:
      //============== Konstruktoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        Constructor with parameter passing.
        \param[in] pParent The owning parent object.
        \param[in] Socket The socket used for communication.
        \exception eut::ErrorNException An exception is thrown if an error occurs.
      \endif

      \if german
        Konstruktor mit Parameterübergabe.
        \param[in] pParent Das besitzende Elternobjekt.
        \param[in] Socket Socket über den kommuniziert wird.
        \exception eut::ErrorNException Bei einem Fehler wird eine Exception geworfen.
      \endif
      */
      Receiver( TcpMain * pParent, SOCKET Socket );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Destructor.
      \endif

      \if german
        Destruktor.
      \endif
      */
      virtual ~Receiver();
      //-----------------------------------------------------------------------

    private:
      //============== Copy-Konstruktor
      //-----------------------------------------------------------------------
      /*!
      \if english
        The copy constructor is not available.
      \endif

      \if german
        Der Copy-Konstruktor steht nicht zur Verfügung.
      \endif
      */
      Receiver( const Receiver & );
      //-----------------------------------------------------------------------

      //============== Operatoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        The assignment operator is not available.
      \endif

      \if german
        Der Zuweisungsoperator steht nicht zur Verfügung.
      \endif
      */
      Receiver & operator= ( const Receiver & );
      //-----------------------------------------------------------------------
  };
  //---------------------------------------------------------------------------
} // namespace tcp
//-----------------------------------------------------------------------------

#endif
