//-----------------------------------------------------------------------------
/*!
\file TcpMessage.cpp
\brief Source for AbstractMessage, Message, ServerAcceptMessage and ServerMessage.
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTcpLib module.

EuTcpLib is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "TcpMessage.h"
#include "EuException.h"
#include <new>
#include <sstream>
#include "EuBlob.h"
#include "EuStdFunctions.h"
#if defined(__GNUC__)
  #include <stdlib.h>
  #include <unistd.h>
#endif
//-----------------------------------------------------------------------------
#ifdef _DEBUG_MEMORY_SPY
  #include <MilDebugNew.h>
#endif
//-----------------------------------------------------------------------------
#if defined(_MSC_VER) || defined(__MINGW32__) || defined(__MINGW64__)
  /*
   * WinSock 2 extension -- manifest constants for shutdown()
   */
  #define SD_RECEIVE      0x00
  #define SD_SEND         0x01
  #define SD_BOTH         0x02
#endif
//-----------------------------------------------------------------------------
namespace tcp
{
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // 't_BadbAllocEx': Unreferenzierte lokale Variable
    #pragma warning( disable : 4101 )
  #endif
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  // >>>>>>>>>>>>>>>>>>>>>>>>> AbstractMessage >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
  //---------------------------------------------------------------------------
  AbstractMessage::AbstractMessage()
  {
  }
  //---------------------------------------------------------------------------
  AbstractMessage::~AbstractMessage()
  {
  }
  //---------------------------------------------------------------------------
  // <<<<<<<<<<<<<<<<<<<<<<<<< AbstractMessage <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  // >>>>>>>>>>>>>>>>>>>>>>>>>>> Message >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
  //---------------------------------------------------------------------------
  #define SUPER AbstractMessage
  //---------------------------------------------------------------------------
  Message::Message()
  : SUPER(),
    m_uiMessageID(0),
    m_iActionCode(0),
    m_nRawMessageSize(0),
    m_pszRawMessage( NULL_PTR ),
    m_nTransmitDataSize(0),
    m_pszTransmitData( NULL_PTR )
  {
  }
  //---------------------------------------------------------------------------
  Message::Message( char * pszTransmitData, std::size_t nTransmitDataSize )
  : SUPER(),
    m_uiMessageID(0),
    m_iActionCode(0),
    m_nRawMessageSize(0),
    m_pszRawMessage( NULL_PTR ),
    m_nTransmitDataSize( nTransmitDataSize ),
    m_pszTransmitData( pszTransmitData )
  {
    int  t_iErr = 0;

    t_iErr = this->initDataFromSendMsg();
    if ( t_iErr )
    {
      m_pszTransmitData = NULL_PTR;
      throw eut::ErrorNException( "Message ctor fehlgeschlagen!", t_iErr, __FILE__, __LINE__ );
    }
  }
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // warning C4189: "t_iDEBUG_XXX": Lokale Variable ist initialisiert aber nicht referenziert
    #pragma warning( disable : 4189 )
  #endif
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: unused variable 't_iDEBUG_XXX' [-Wunused-variable]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wunused-variable"
  #endif
  //---------------------------------------------------------------------------
  Message::Message( char * pszRawMsg,
                    std::size_t nRawMsgSize,
                    unsigned int uiMessageID,
                    int iActionCode,
                    const std::string & rstrInfo)
  : SUPER(),
    m_uiMessageID( uiMessageID ),
    m_iActionCode( iActionCode ),
    m_strInfo( rstrInfo ),
    m_nRawMessageSize( nRawMsgSize ),
    m_pszRawMessage( pszRawMsg ),
    m_nTransmitDataSize(0),
    m_pszTransmitData( NULL_PTR )
  {
    int  t_iErr = 0;

    #ifdef _DEBUG
      if ( m_iActionCode == ERR_WSAECONNRESET )
      {
        int t_iDEBUG_BreakPoint = 1;
      }
    #endif
    t_iErr = this->initSendMsgFromData();
    if ( t_iErr )
    {
      m_pszTransmitData = NULL_PTR;
      throw eut::ErrorNException( "Message ctor fehlgeschlagen!", t_iErr, __FILE__, __LINE__ );
    }
  }
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // warning C4189: "t_iDEBUG_XXX": Lokale Variable ist initialisiert aber nicht referenziert
    #pragma warning( default : 4189 )
  #endif
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: unused variable 't_iDEBUG_XXX' [-Wunused-variable]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  Message::~Message()
  {
    if ( m_pszTransmitData )
    {
      delete[] m_pszTransmitData;
    }
  }
  //---------------------------------------------------------------------------
  Message::Message( const Message & )
  : m_uiMessageID(0),
    m_iActionCode(0),
    m_nRawMessageSize(0),
    m_pszRawMessage( NULL_PTR ),
    m_nTransmitDataSize(0),
    m_pszTransmitData( NULL_PTR )
  {
  }
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // warning C4189: "t_iDEBUG_XXX": Lokale Variable ist initialisiert aber nicht referenziert
    #pragma warning( disable : 4189 )
  #endif
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: unused variable 't_iDEBUG_XXX' [-Wunused-variable]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wunused-variable"
  #endif
  //---------------------------------------------------------------------------
  int Message::initDataFromSendMsg()
  {
    int  t_iErr = 0;

    if ( m_pszTransmitData && m_nTransmitDataSize > 0 )
    {
      char            t_szKey[] = "hja-tcp";
      // ist das eine Msg, welche uns betrifft?
      if ( strcmp( t_szKey, m_pszTransmitData ) != 0 ) return ERR_WRONG_STREAMTYPE;

      std::size_t   t_nBlobPosition = 0,
                    t_nBlobCurrentPosition = 0;
      eut::Blob     t_Blob( m_pszTransmitData, m_nTransmitDataSize, false );

      // Indentifikator überholen
      if ( ! t_Blob.findForward( t_nBlobPosition, "\0", 1, true ) ) t_iErr = ERR_NO_CORRECT_STREAM_DATA;
      if ( ! t_iErr )
      {
        t_nBlobCurrentPosition = t_nBlobPosition;
        if ( m_nTransmitDataSize < t_nBlobCurrentPosition ) t_iErr = ERR_NO_CORRECT_STREAM_DATA;
      }
      // Länge mit Länge aus Daten vergleichen
      if ( ! t_iErr ) if ( ! t_Blob.findForwardNext( t_nBlobPosition, "\0", 1 ) ) t_iErr = ERR_NO_CORRECT_STREAM_DATA;
      if ( ! t_iErr )
      {
        unsigned int    t_uiComparisonSize = 0;

        std::istringstream( m_pszTransmitData + t_nBlobCurrentPosition ) >> t_uiComparisonSize;

        if ( t_uiComparisonSize != m_nTransmitDataSize ) t_iErr = ERR_NO_CORRECT_STREAM_DATA;
        if ( ! t_iErr )
        {
          t_nBlobCurrentPosition = t_nBlobPosition;
          if ( m_nTransmitDataSize < t_nBlobCurrentPosition )
          {
            t_iErr = ERR_NO_CORRECT_STREAM_DATA;
          }
        }
      }
      // Msg-ID holen, ist 0-terminiertes Element
      if ( ! t_iErr ) if ( ! t_Blob.findForwardNext( t_nBlobPosition, "\0", 1 ) ) t_iErr = ERR_NO_CORRECT_STREAM_DATA;
      if ( ! t_iErr )
      {
        std::istringstream( m_pszTransmitData + t_nBlobCurrentPosition ) >> m_uiMessageID;
        t_nBlobCurrentPosition = t_nBlobPosition;
        if ( m_nTransmitDataSize < t_nBlobCurrentPosition ) t_iErr = ERR_NO_CORRECT_STREAM_DATA;
      }
      // Aktionscode holen, ist 0-terminiertes Element
      if ( ! t_iErr ) if ( ! t_Blob.findForwardNext( t_nBlobPosition, "\0", 1 ) ) t_iErr = ERR_NO_CORRECT_STREAM_DATA;
      if ( ! t_iErr )
      {
        std::istringstream( m_pszTransmitData + t_nBlobCurrentPosition ) >> m_iActionCode;
        #ifdef _DEBUG
          if ( m_iActionCode == ERR_WSAECONNRESET )
          {
            int t_iDEBUG_BreakPoint = 1;
          }
        #endif
        t_nBlobCurrentPosition = t_nBlobPosition;
        if ( m_nTransmitDataSize < t_nBlobCurrentPosition ) t_iErr = ERR_NO_CORRECT_STREAM_DATA;
      }
      // Infosstring holen, ist 0-terminiertes Element
      if ( ! t_iErr ) if ( ! t_Blob.findForwardNext( t_nBlobPosition, "\0", 1 ) ) t_iErr = ERR_NO_CORRECT_STREAM_DATA;
      if ( ! t_iErr )
      {
        m_strInfo = m_pszTransmitData + t_nBlobCurrentPosition;
        t_nBlobCurrentPosition = t_nBlobPosition;
        if (  m_nTransmitDataSize < t_nBlobCurrentPosition ) t_iErr = ERR_NO_CORRECT_STREAM_DATA;
      }
      // RawMsgSize ermitteln
      m_nRawMessageSize = m_nTransmitDataSize - t_nBlobCurrentPosition;
      // und RawMsg in Pointer
      m_pszRawMessage = m_pszTransmitData + t_nBlobCurrentPosition;
      t_Blob.setNULL();
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // warning C4189: "t_iDEBUG_XXX": Lokale Variable ist initialisiert aber nicht referenziert
    #pragma warning( default : 4189 )
  #endif
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: unused variable 't_iDEBUG_XXX' [-Wunused-variable]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // warning C4267: "+=": Konvertierung von "size_t" nach "unsigned int", Datenverlust möglich
    #pragma warning( disable : 4267 )
  #endif
  //---------------------------------------------------------------------------
  int Message::initSendMsgFromData()
  {
    int                    t_iErr( 0 );
    eut::Blob             t_HeaderBlob;
    std::ostringstream    t_ossConverter;
    std::string           t_strMessageID,
                          t_strActionID,
                          t_strTransmitDataSize;
    char                  t_szKey[] = "hja-tcp";
    unsigned int          t_uiTransmitDataSize( 0 ),
                          t_uiHeaderSize( 8 ); // Größe von t_szKey mit 0

    // Größe der zu sendenden Daten berechnen, Msg-ID
    t_ossConverter.str("");
    t_ossConverter << m_uiMessageID;
    t_strMessageID = t_ossConverter.str();
    t_uiHeaderSize += t_strMessageID.size() + 1;
    // ActionCode
    t_ossConverter.str("");
    t_ossConverter << m_iActionCode;
    t_strActionID = t_ossConverter.str();
    t_uiHeaderSize += t_strActionID.size() + 1;
    // InfoString
    t_uiHeaderSize += m_strInfo.size() + 1;
    // Raw-Daten
    t_uiTransmitDataSize += m_nRawMessageSize;
    // die TransmitDataSize ist immer 10 stellig mit terminierender 0
    t_uiHeaderSize += 11;
    t_uiTransmitDataSize += t_uiHeaderSize;
    t_ossConverter.str("");
    t_ossConverter << t_uiTransmitDataSize;
    t_strTransmitDataSize = t_ossConverter.str();
    eut::StdStringFunctions::fillString( t_strTransmitDataSize, '0', 10, eut::StdStringFunctions::eBEFORE );

    // Identifikator 0-terminiert in HeaderBlob
    t_iErr = t_HeaderBlob.insert( t_szKey, 7, t_HeaderBlob.getSize() + 1 );
    if ( ! t_iErr ) t_iErr = t_HeaderBlob.insert( "\0", 1, t_HeaderBlob.getSize() + 1 );
    // TransmitDataSize 0-terminiert in HeaderBlob
    if ( ! t_iErr ) t_iErr = t_HeaderBlob.insert( t_strTransmitDataSize.c_str(), t_strTransmitDataSize.size(), t_HeaderBlob.getSize() + 1 );
    if ( ! t_iErr ) t_iErr = t_HeaderBlob.insert( "\0", 1, t_HeaderBlob.getSize() + 1 );
    // Msg-ID 0-terminiert in HeaderBlob
    if ( ! t_iErr ) t_iErr = t_HeaderBlob.insert( t_strMessageID.c_str(), t_strMessageID.size(), t_HeaderBlob.getSize() + 1 );
    if ( ! t_iErr ) t_iErr = t_HeaderBlob.insert( "\0", 1, t_HeaderBlob.getSize() + 1 );
    // Actionscode 0-terminiert in HeaderBlob
    if ( ! t_iErr ) t_iErr = t_HeaderBlob.insert( t_strActionID.c_str(), t_strActionID.size(), t_HeaderBlob.getSize() + 1 );
    if ( ! t_iErr ) t_iErr = t_HeaderBlob.insert( "\0", 1, t_HeaderBlob.getSize() + 1 );
    // InfoString 0-terminiert in HeaderBlob
    if ( ! t_iErr && ! m_strInfo.empty() ) t_iErr = t_HeaderBlob.insert( m_strInfo.c_str(), m_strInfo.size(), t_HeaderBlob.getSize() + 1 );
    if ( ! t_iErr ) t_iErr = t_HeaderBlob.insert( "\0", 1, t_HeaderBlob.getSize() + 1 );
    // Raw-Daten in HeaderBlob
    if ( ! t_iErr && m_nRawMessageSize > 0 )
    {
      t_iErr = t_HeaderBlob.insert( m_pszRawMessage, m_nRawMessageSize, t_HeaderBlob.getSize() + 1 );
    }
    if ( ! t_iErr )
    {
      if ( t_uiTransmitDataSize != t_HeaderBlob.getSize() )
      {
        throw eut::ErrorNException( "Message::initSendMsgFromData() fehlgeschlagen!",
                                    t_iErr,
                                    __FILE__,
                                    __LINE__ );
      }
      m_nTransmitDataSize = t_HeaderBlob.getData( m_pszTransmitData, false );
      t_HeaderBlob.setNULL();
      if ( m_pszRawMessage )
      {
        delete[] m_pszRawMessage;
        m_pszRawMessage = NULL_PTR;
      }
      if ( m_nRawMessageSize )
      {
        m_pszRawMessage = m_pszTransmitData + t_uiHeaderSize;
      }
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // warning C4267: "+=": Konvertierung von "size_t" nach "unsigned int", Datenverlust möglich
    #pragma warning( default : 4267 )
  #endif
  //---------------------------------------------------------------------------
  char * Message::getRawMsgPt()
  {
    return m_pszRawMessage;
  }
  //---------------------------------------------------------------------------
  std::size_t Message::getRawMsgSize() const
  {
    return m_nRawMessageSize;
  }
  //---------------------------------------------------------------------------
  char * Message::getTransmitDataPt()
  {
    return m_pszTransmitData;
  }
  //---------------------------------------------------------------------------
  std::size_t Message::getTransmitDataSize() const
  {
    return m_nTransmitDataSize;
  }
  //---------------------------------------------------------------------------
  unsigned int Message::getMsgID() const
  {
    return m_uiMessageID;
  }
  //---------------------------------------------------------------------------
  int Message::getActionCode() const
  {
    return m_iActionCode;
  }
  //---------------------------------------------------------------------------
  std::string Message::getInfo() const
  {
    return m_strInfo;
  }
  //---------------------------------------------------------------------------
  std::size_t Message::getRawMsg( char *& prszRawMsg, bool bCopy )
  {
    if ( m_pszRawMessage && m_nRawMessageSize )
    {
      if ( bCopy )
      {
      try
      {
        prszRawMsg = new char [m_nRawMessageSize];
      }
      catch ( const std::bad_alloc & )
      {
        throw eut::ErrorNException( "Message::getRawMsg(char *& prszRawMsg, bool bCopy)",
                                    eut::ERR_NEW,
                                    __FILE__,
                                    __LINE__ );
      }
        std::memcpy( prszRawMsg, m_pszRawMessage,  m_nRawMessageSize);
      }
      else
      {
        prszRawMsg = m_pszRawMessage;
      }
    }
    return m_nRawMessageSize;
  }
  //---------------------------------------------------------------------------
  std::size_t Message::getTransmitData( char *& prszSendMsg, bool bCopy )
  {
    if ( m_pszTransmitData && m_nTransmitDataSize )
    {
      if ( bCopy )
      {
      try
      {
        prszSendMsg = new char [m_nTransmitDataSize];
      }
      catch ( const std::bad_alloc & )
      {
      throw eut::ErrorNException( "Message::getTransmitData(char *& prszSendMsg, bool bCopy)",
                                  eut::ERR_NEW,
                                  __FILE__,
                                  __LINE__ );
      }
        memcpy( prszSendMsg, m_pszTransmitData,  m_nTransmitDataSize);
      }
      else
      {
        prszSendMsg = m_pszTransmitData;
      }
    }
    return m_nTransmitDataSize;
  }
  //---------------------------------------------------------------------------
  Message & Message::operator= ( const Message & )
  {
    return *this;
  }
  //---------------------------------------------------------------------------
  #undef SUPER
  //---------------------------------------------------------------------------
  // <<<<<<<<<<<<<<<<<<<<<<<<<<<< Message <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  // >>>>>>>>>>>>>>>>>>>>>>>> ServerAcceptMessage >>>>>>>>>>>>>>>>>>>>>>>>>>>>>
  //---------------------------------------------------------------------------
  #define SUPER AbstractMessage
  //---------------------------------------------------------------------------
  ServerAcceptMessage::ServerAcceptMessage()
  : SUPER(),
    m_ClientSocket(0)
  {
  }
  //---------------------------------------------------------------------------
  ServerAcceptMessage::ServerAcceptMessage( SOCKET NewSocket )
  : SUPER(),
    m_ClientSocket( NewSocket )
  {
  }
  //---------------------------------------------------------------------------
  ServerAcceptMessage::~ServerAcceptMessage()
  {
  }
  //---------------------------------------------------------------------------
  ServerAcceptMessage::ServerAcceptMessage( const ServerAcceptMessage & )
  : m_ClientSocket(0)
  {
  }
  //---------------------------------------------------------------------------
  SOCKET ServerAcceptMessage::getSocket()
  {
      return m_ClientSocket;
  }
  //---------------------------------------------------------------------------
  #if defined (__clang__)
    // Warnung: implicit conversion loses integer precision: A to B [-Wshorten-64-to-32]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wshorten-64-to-32"
  #endif
  //---------------------------------------------------------------------------
  void ServerAcceptMessage::closeSocket()
  {
    #if defined(_MSC_VER) || defined(__MINGW32__)
      shutdown( m_ClientSocket, SD_BOTH );
      closesocket( m_ClientSocket );
    #elif defined(__GNUC__)
      shutdown( m_ClientSocket, SHUT_RDWR );
      close( m_ClientSocket );
    #endif
    m_ClientSocket = 0;
  }
  //---------------------------------------------------------------------------
  #if defined (__clang__)
    // Warnung: implicit conversion loses integer precision: A to B [-Wshorten-64-to-32]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  ServerAcceptMessage & ServerAcceptMessage::operator= ( const ServerAcceptMessage & )
  {
    return *this;
  }
  //---------------------------------------------------------------------------
  #undef SUPER
  //---------------------------------------------------------------------------
  // <<<<<<<<<<<<<<<<<<<<<<<<< ServerAcceptMessage <<<<<<<<<<<<<<<<<<<<<<<<<<<<
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  // >>>>>>>>>>>>>>>>>>>>>>>>>>> ServerMessage >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
  //---------------------------------------------------------------------------
  #define SUPER_1 Message
  #define SUPER_2 ServerAcceptMessage
  //---------------------------------------------------------------------------
  ServerMessage::ServerMessage()
  : SUPER_1(),
    SUPER_2()
  {
  }
  //---------------------------------------------------------------------------
  ServerMessage::ServerMessage( char * pszTransmitData, std::size_t nTransmitDataSize )
  : SUPER_1( pszTransmitData, nTransmitDataSize ),
    SUPER_2()
  {
  }
  //---------------------------------------------------------------------------
  ServerMessage::ServerMessage( SOCKET ClientSocket,
                                char * pszTransmitData,
                                std::size_t nTransmitDataSize)
  : SUPER_1( pszTransmitData, nTransmitDataSize ),
    SUPER_2( ClientSocket )
  {
  }
  //---------------------------------------------------------------------------
  ServerMessage::ServerMessage( SOCKET ClientSocket,
                                char * pszRawMsg,
                                std::size_t nRawMsgSize,
                                unsigned int uiMessageID,
                                int iActionCode,
                                const std::string & rstrInfo)
  : SUPER_1( pszRawMsg, nRawMsgSize, uiMessageID, iActionCode, rstrInfo ),
    SUPER_2( ClientSocket )
  {
  }
  //---------------------------------------------------------------------------
  ServerMessage::~ServerMessage()
  {
  }
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: base class 'class Message' should be explicitly initialized in the copy constructor [-Wextra]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wextra"
  #endif
  //---------------------------------------------------------------------------
  ServerMessage::ServerMessage( const ServerMessage & )
  {
  }
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: base class 'class Message' should be explicitly initialized in the copy constructor [-Wextra]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  ServerMessage & ServerMessage::operator= ( const ServerMessage & )
  {
    return *this;
  }
  //---------------------------------------------------------------------------
  // <<<<<<<<<<<<<<<<<<<<<<<<<<< ServerMessage <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  //---------------------------------------------------------------------------
  #undef SUPER_1
  #undef SUPER_2
  //---------------------------------------------------------------------------

  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // 't_BadbAllocEx': Unreferenzierte lokale Variable
    #pragma warning( default : 4101 )
  #endif
  //---------------------------------------------------------------------------
} // namespace tcp
//-----------------------------------------------------------------------------
