//-----------------------------------------------------------------------------
/*!
\file TcpServerAcceptReceiver.cpp
\brief ServerAcceptReceiver.
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTcpLib module.

EuTcpLib is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "TcpServerAcceptReceiver.h"
#include "TcpServer.h"
#if defined (__GNUC__) && ! defined (__MINGW32__)
  #include <stdlib.h>
  #include <unistd.h>
  #define WSAENOTSOCK 10038L
  #define WSAECONNRESET 10054L
#endif
#ifdef _DEBUG
  #include <iostream>
  #include <sstream>
#endif
//-----------------------------------------------------------------------------
#ifdef _DEBUG_MEMORY_SPY
  #include <MilDebugNew.h>
#endif
//-----------------------------------------------------------------------------
#if defined(_MSC_VER) || defined(__MINGW32__) || defined(__MINGW64__)
  /*
   * WinSock 2 extension -- manifest constants for shutdown()
   */
  #define SD_RECEIVE      0x00
  #define SD_SEND         0x01
  #define SD_BOTH         0x02
#endif
//-----------------------------------------------------------------------------
namespace tcp
{
  //---------------------------------------------------------------------------
  #define SUPER Receiver
  //---------------------------------------------------------------------------
  ServerAcceptReceiver::ServerAcceptReceiver()
  : SUPER()
  {
    #ifdef _DEBUG
      m_strDebugInfo = "ServerAcceptReceiver";
    #endif
  }
  //---------------------------------------------------------------------------
  ServerAcceptReceiver::ServerAcceptReceiver( SOCKET Socket )
  : SUPER( Socket )
  {
    #ifdef _DEBUG
      m_strDebugInfo = "ServerAcceptReceiver";
    #endif
  }
  //---------------------------------------------------------------------------
  ServerAcceptReceiver::ServerAcceptReceiver( TcpMain * pParent, SOCKET Socket )
  : SUPER( pParent, Socket )
  {
    #ifdef _DEBUG
      m_strDebugInfo = "ServerAcceptReceiver";
    #endif
  }
  //---------------------------------------------------------------------------
  ServerAcceptReceiver::ServerAcceptReceiver( Server * pParent, SOCKET Socket )
  : SUPER( pParent, Socket )
  {
    #ifdef _DEBUG
      m_strDebugInfo = "ServerAcceptReceiver";
    #endif
  }
  //---------------------------------------------------------------------------
  ServerAcceptReceiver::~ServerAcceptReceiver()
  {
  }
  //---------------------------------------------------------------------------
  #if defined (__GNUC__)
    // Warnung: base class 'class Receiver' should be explicitly initialized in the copy constructor [-Wextra]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wextra"
  #endif
  //---------------------------------------------------------------------------
  ServerAcceptReceiver::ServerAcceptReceiver( const ServerAcceptReceiver & )
  {
  }
  //---------------------------------------------------------------------------
  #if defined (__GNUC__)
    // Warnung: base class 'class Receiver' should be explicitly initialized in the copy constructor [-Wextra]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  #if defined (__clang__)
    // Warnung: implicit conversion loses integer precision: A to B [-Wshorten-64-to-32]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wshorten-64-to-32"
  #endif
  //---------------------------------------------------------------------------
  #if defined (__GNUC__)
    // Warnung: comparison between signed and unsigned integer expressions [-Wsign-compare]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wsign-compare"
    // Warnung: implicit conversion changes signedness: A to B [-Wsign-conversion]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wsign-conversion"
  #endif
  //---------------------------------------------------------------------------
  void ServerAcceptReceiver::work()
  {
    SOCKET    t_NewConnectSocket = 0;

    #if defined (__PTHREADS)
      m_eThreadStatus = EuPTHREAD_WORKING;
    #endif
    // Blockiert!
    t_NewConnectSocket = accept( m_Socket, NULL_PTR, NULL_PTR );
    #if defined (__PTHREADS)
      pthread_testcancel();
    #else
      if ( this->testCancel() )
      {
        return;
      }
    #endif
    // wenn m_Socket noch da...
    if ( m_Socket )
    {
      int               t_iErr = 0;
      AbstractMessage * t_pTcpAbstractMessage = NULL_PTR;
      Server          * t_pMyParent = dynamic_cast<Server*>( m_pParentTcpMain );
      
      if( t_NewConnectSocket == SOCKET_ERROR)
      {
        std::string   t_strErrMsg( "AcceptErr" );
        #if defined(_MSC_VER) || defined(__MINGW32__)
          int         t_iActionCode =  WSAGetLastError();
        #elif defined(__GNUC__)
          int         t_iActionCode = errno;
        #endif

        t_pTcpAbstractMessage = new Message(  NULL_PTR,       // RawMsg
                                              0,              // RawMsgSize
                                              0,              // MsgID
                                              t_iActionCode,  // ActionCode
                                              t_strErrMsg );  // Info
        #ifdef _DEBUG_TcpMessage
          t_pTcpAbstractMessage->m_sDebugInstanceInfo = "ServerAcceptReceiver::work()";
        #endif
        // wird im Parent wieder gelöscht!
        t_iErr = t_pMyParent->acceptClientMessage( t_pTcpAbstractMessage );
      }
      else
      {
        t_pTcpAbstractMessage = new ServerAcceptMessage( t_NewConnectSocket );
        #ifdef _DEBUG_TcpMessage
          t_pTcpAbstractMessage->m_sDebugInstanceInfo = "ServerAcceptReceiver::work()";
        #endif
        // wird im Parent wieder gelöscht!
        t_iErr = t_pMyParent->acceptClientMessage( t_pTcpAbstractMessage );
      }
      if ( t_iErr )
      {
        delete t_pTcpAbstractMessage;
      }
    }
  }
  //---------------------------------------------------------------------------
  #if defined (__GNUC__)
    // Warnung: comparison between signed and unsigned integer expressions [-Wsign-compare]
    #pragma GCC diagnostic pop
    // Warnung: implicit conversion changes signedness: A to B [-Wsign-conversion]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  #if defined (__clang__)
    // Warnung: implicit conversion loses integer precision: A to B [-Wshorten-64-to-32]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  #if defined (__PTHREADS)
  void ServerAcceptReceiver::cleanUp()
  {
    SUPER::cleanUp();
  }
  #endif
  //---------------------------------------------------------------------------
  #if defined (__clang__)
    // Warnung: implicit conversion loses integer precision: A to B [-Wshorten-64-to-32]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wshorten-64-to-32"
  #endif
  //---------------------------------------------------------------------------
  #ifdef __PTHREADS
    int ServerAcceptReceiver::cancel()
  #else
    void ServerAcceptReceiver::cancel()
  #endif
  {
    if ( m_Socket )
    {
      #if defined(_MSC_VER) || defined(__MINGW32__)
        shutdown( m_Socket, SD_BOTH );
        closesocket( m_Socket );
      #elif defined(__GNUC__)
        shutdown( m_Socket, SHUT_RDWR );
        close( m_Socket );
      #endif
      m_Socket = 0;
    }
    #ifdef __PTHREADS
      return SUPER::cancel();
    #else
      SUPER::cancel();
    #endif
  }
  //---------------------------------------------------------------------------
  #if defined (__clang__)
    // Warnung: implicit conversion loses integer precision: A to B [-Wshorten-64-to-32]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  ServerAcceptReceiver & ServerAcceptReceiver::operator= ( const ServerAcceptReceiver & )
  {
    return *this;
  }
  //---------------------------------------------------------------------------
  #undef SUPER
  //---------------------------------------------------------------------------
} // namespace tcp
//-----------------------------------------------------------------------------
