#ifndef INC_EUDEF_NULL_PTR_H
#define INC_EUDEF_NULL_PTR_H

//-----------------------------------------------------------------------------
/*!
\file EuDef_NULL_PTR.h
\brief
\if english
  This file exists so that the definition of NULL_PTR is centrally available. 

  This file is usually included by every header where NULL_PTR is used.
\endif

\if german
  Damit die Definition von NULL_PTR und NUM_PTR zentral verfügbar ist, gibt es
  diese Datei. 

  Diese Datei wird in der Regel von jeder Header, wo NULL_PTR bzw. NUM_PTR
  genutzt wird, eingebunden.
\endif
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTools module.

EuTools is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
/*! \anchor NULL_PTR
  \if english
    Depending on the C++ version <b>NULL_PTR</b> is defined as <b>nullptr</b> or <b>NULL</b>.
  \endif

  \if german
    Je nach C++ Version wird <b>NULL_PTR</b> als <b>nullptr</b> oder <b>NULL</b> definiert.
  \endif
  \code{.cpp}
  #if __cplusplus >= 201103L
    #define NULL_PTR nullptr
  #else
    #define NULL_PTR NULL
  #endif // __cplusplus >= 201703L
  \endcode
  
  \if english
    In Visual Studio extend the "Properties::C/C++::All options::Additional options:"
    with "/Zc:__cplusplus"!
  \endif

  \if german
    In Visual Studio die "Eigenschaften::C/C++::Alle Optionen::Zusätzliche Optionen:"
    um "/Zc:__cplusplus" erweitern!
  \endif
*/
#ifndef NULL_PTR
  #if __cplusplus >= 201103L
    #define NULL_PTR nullptr
  #else
    #define NULL_PTR NULL
  #endif // __cplusplus >= 201703L
#endif
//-----------------------------------------------------------------------------
/*!
\anchor NUM_PTR
\if english
  To support 32- and 64-bit, the type <b>NUM_PTR</b> is introduced for pointers
  to functions. This is always <b>long long</b>; i.e. <b>64-bit</b> large. 
\endif

\if german
  Um 32- und 64-Bit zu unterstützen, wird für Zeiger auf Funktionen der Typ
  <b>NUM_PTR</b> eingeführt. Dieser ist immer <b>long long</b>; also <b>64-Bit</b> groß. 
\endif
  \code{.cpp}
  typedef long long NUM_PTR;
  \endcode
*/
typedef long long NUM_PTR;
//-----------------------------------------------------------------------------
/*!
\if english
  Settings for C++ versions < 11. (201103L)
\endif

\if german
  Einstellungen für C++ Versionen < 11 (201103L).
\endif
  #ifdef _MSC_VER
    #if __cplusplus < 201103L
    // warning C4189: "t_DEBUG_XXX": Lokale Variable ist initialisiert aber nicht referenziert
    #pragma warning( disable : 4189 )
    #endif // __cplusplus < 201703L
  #endif
*/
#ifdef _MSC_VER
  #if __cplusplus < 201103L
  // warning C4481: Nicht dem Standard entsprechende Erweiterung: Spezifizierer "override" überschreiben.
  #pragma warning( disable : 4481 )
  #endif // __cplusplus < 201703L
#endif
//-----------------------------------------------------------------------------
#endif
