//-----------------------------------------------------------------------------
/*!
\file EuCriticalSection.cpp
\brief Source for CriticalSection
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuPThreads module.

EuPThreads is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "EuCriticalSection.h"
//-----------------------------------------------------------------------------
namespace eut
{
  //---------------------------------------------------------------------------
  CriticalSection::CriticalSection ()
  : m_bIsLocked( false )
  {
  }
  //---------------------------------------------------------------------------
  CriticalSection::~CriticalSection()
  {
  }
  //---------------------------------------------------------------------------
  CriticalSection::CriticalSection( const CriticalSection & )
  : m_bIsLocked( false )
  {
  }
  //---------------------------------------------------------------------------
  void CriticalSection::lock()
  {
    m_ExclusiveAccessMutex.lock();
    m_bIsLocked = true;
  }
  //---------------------------------------------------------------------------
  void CriticalSection::unlock()
  {
    m_ExclusiveAccessMutex.unlock();
    m_bIsLocked = false;
  }
  //---------------------------------------------------------------------------
  bool CriticalSection::isLocked() const
  {
    return m_bIsLocked;
  }
  //---------------------------------------------------------------------------
  CriticalSection & CriticalSection::operator= ( const CriticalSection & )
  {
    m_bIsLocked = false;
    return *this;
  }
  //---------------------------------------------------------------------------
} // namespace eut
